'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DETECTION_DELAY = 5000;

var activationHandlers = {};

var optimizelyPromise = void 0;

var wasExperimentsListenerAdded = false;

/**
 * Waits until func returns a value and calls callback with it
 * @param  {Function}   truthyFn       Function to execute to check value
 * @param  {Function} callback       Callback called passing value
 * @param  {Number}   [delay=100]    Delay allowed to wait for value
 * @param  {Number}   [interval=100] Interval time for checks
 */
var waitUntil = function waitUntil(truthyFn, callback) {
  var delay = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 100;
  var interval = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 100;

  var intervalId = setInterval(function () {
    var value = truthyFn();
    if (value || delay <= 0) {
      clearInterval(intervalId);
      callback(value);
    }
    delay -= interval;
  }, interval);
};

var getOptimizely = function getOptimizely() {
  return window && window.optimizely && window.optimizely.get && window.optimizely;
};

/**
 * Register handler to optimizely ONCE
 * @param {Object} sdk OptimizelySdk
 */
var registerExprimentsEvents = function registerExprimentsEvents(sdk) {
  if (sdk && !wasExperimentsListenerAdded) {
    sdk.push({
      type: 'addListener',
      filter: { type: 'lifecycle', name: 'campaignDecided' },
      handler: experimentsActivationHandler
    });
    wasExperimentsListenerAdded = true;
  }
};

/**
 * Handler for Optimizely events
 * @param {Object} event
 */
var experimentsActivationHandler = function experimentsActivationHandler(event) {
  var _event$data$decision = event.data.decision,
      experimentId = _event$data$decision.experimentId,
      variationId = _event$data$decision.variationId;

  dispatchActivated(experimentId, variationId);
};

/**
 * Executed all handlers added to an experiment with variationId as param
 * @param {String} experimentId
 * @param {String} variationId
 */
var dispatchActivated = function dispatchActivated(experimentId, variationId) {
  var handlers = activationHandlers[experimentId] || [];
  handlers.forEach(function (handler) {
    return handler(variationId);
  });
};

var OptimizelyXExperiments = function () {
  function OptimizelyXExperiments() {
    (0, _classCallCheck3.default)(this, OptimizelyXExperiments);
  }

  (0, _createClass3.default)(OptimizelyXExperiments, null, [{
    key: 'getSDK',

    /**
     * Get OptimizelyX SDK instance
     * @return Promise<Object>
     */
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                optimizelyPromise = optimizelyPromise || new _promise2.default(function (resolve) {
                  waitUntil(getOptimizely, function (sdk) {
                    registerExprimentsEvents(sdk);
                    resolve(sdk);
                  }, DETECTION_DELAY);
                });
                return _context.abrupt('return', optimizelyPromise);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function getSDK() {
        return _ref.apply(this, arguments);
      }

      return getSDK;
    }()

    /**
     * Get info of an experiment
     * @param {String} experimentId
     * @return Promise<Object>
     * @example
     * {
     *  id: '100',
     *  isActive: true,
     *  isInExperimentHoldback: true,
     *  variation: {
     *      id: '101',
     *      name: 'Variation #1'
     *  }
     * }
     */

  }, {
    key: 'getInfo',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(experimentId) {
        var sdk;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this.getSDK();

              case 2:
                sdk = _context2.sent;
                return _context2.abrupt('return', sdk && sdk.get && sdk.get('state').getExperimentStates()[experimentId]);

              case 4:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function getInfo(_x3) {
        return _ref2.apply(this, arguments);
      }

      return getInfo;
    }()

    /**
     * Get if an experiment is activated for current user
     * @param {String} experimentId
     * @return Promise<Boolean>
     */

  }, {
    key: 'isActivated',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(experimentId) {
        var info;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this.getInfo(experimentId);

              case 2:
                _context3.t0 = _context3.sent;

                if (_context3.t0) {
                  _context3.next = 5;
                  break;
                }

                _context3.t0 = {};

              case 5:
                info = _context3.t0;
                return _context3.abrupt('return', info.isActive && !info.isInExperimentHoldback || false);

              case 7:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function isActivated(_x4) {
        return _ref3.apply(this, arguments);
      }

      return isActivated;
    }()

    /**
     * Add function to call when experiment gets activated.
     * If experiment is already activated, the handler is called right away.
     * @param {String} experimentId
     * @param {Function} handler
     */

  }, {
    key: 'addActivationListener',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(experimentId, handler) {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                activationHandlers[experimentId] = activationHandlers[experimentId] || [];
                activationHandlers[experimentId].push(handler);
                _context4.next = 4;
                return this.isActivated(experimentId);

              case 4:
                if (!_context4.sent) {
                  _context4.next = 10;
                  break;
                }

                _context4.t0 = handler;
                _context4.next = 8;
                return this.getVariation(experimentId);

              case 8:
                _context4.t1 = _context4.sent;
                (0, _context4.t0)(_context4.t1);

              case 10:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function addActivationListener(_x5, _x6) {
        return _ref4.apply(this, arguments);
      }

      return addActivationListener;
    }()

    /**
     * Remove function to call when experiment gets activated.
     * @param {String} experimentId
     * @param {Function} removedHandler
     */

  }, {
    key: 'removeActivationListener',
    value: function () {
      var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(experimentId, removedHandler) {
        var handlers;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                handlers = activationHandlers[experimentId];

                if (handlers) {
                  _context5.next = 3;
                  break;
                }

                return _context5.abrupt('return');

              case 3:
                activationHandlers[experimentId] = handlers.filter(function (handler) {
                  return handler !== removedHandler;
                });

              case 4:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function removeActivationListener(_x7, _x8) {
        return _ref5.apply(this, arguments);
      }

      return removeActivationListener;
    }()

    /**
     * Get variation to display for current user
     * @param {String} experimentId
     * @return Promise<String|Number>
     */

  }, {
    key: 'getVariation',
    value: function () {
      var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(experimentId) {
        var info;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return this.getInfo(experimentId);

              case 2:
                _context6.t0 = _context6.sent;

                if (_context6.t0) {
                  _context6.next = 5;
                  break;
                }

                _context6.t0 = {};

              case 5:
                info = _context6.t0;
                return _context6.abrupt('return', info.variation.id || null);

              case 7:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function getVariation(_x9) {
        return _ref6.apply(this, arguments);
      }

      return getVariation;
    }()
  }]);
  return OptimizelyXExperiments;
}();

exports.default = OptimizelyXExperiments;